#!/usr/bin/perl -w
use strict;

use Sys::Hostname  qw(hostname);
use LWP::UserAgent ();
use Digest::MD5    ();

# So we don't tag the server(s) all at once
sleep int rand(60*60*4);

my $rpm_list = `yum list | grep FON`;

my ($user, $pass) = get_auth();
my $sid           = get_sid();
my $info          = get_rpm_info();

# If we have no packages out of sync, exit
if (ref($info) ne 'HASH' || keys(%$info) < 1)
{
	exit;
}

post_url($sid, $user, $pass, $info);

sub post_url
{
	my ($server_id, $user, $passwd, $pkgs) = @_;
	my $url = $server_id < 100_000 ? 'http://cp.fonality.com/' : 'http://cp.trixbox.com/';
	$url   .= "update_check.cgi?server_id=$server_id&user=$user&hash=$passwd";

	foreach my $pkg (keys %$pkgs)
	{
		$url .= "\&$pkg=$pkgs->{$pkg}{version}";
		if (exists($pkgs->{$pkg}{reason}) && $pkgs->{$pkg}{reason} eq 'invalid')
		{
			$url .= ',invalid';
		}
	}

	my $lwp_request = LWP::UserAgent->new();
	$lwp_request->get($url);
}

# construct the info; ignore "is not installed" warnings since they may not have gotten the upgrade to install a particular package
sub get_rpm_info
{
	my %info;
	foreach my $package (split /\n/, $rpm_list)
	{
		$package =~ s/^(\w+)\..*/$1/;
		my $version = `rpm -q $package`;
		chomp($version);

		if ($version !~ /is not installed/)
		{
			$version = join '-', (split /-/, $version)[1,2]; #~ s/^[^-]-([\d-]*)$/$1/;
			$info{$package} = { version => $version };
		}
		else
		{
			next;
		}

		my $oops = `rpm -V $package`;
		chomp($oops);

		# If there's a response that means there's a difference. If the response is "is not installed", ignore it.
		if ($oops && $oops !~ /is not installed/)
		{
			$info{$package}{reason} = 'invalid';
		}
	}

	return wantarray ? %info : \%info;
}

sub get_sid
{
	my ($server_id) = hostname() =~ /\A \D* (\d+) \z/xms;
	return $server_id;
}

sub get_auth
{
	my ($user, $pass) = ('', '');

	open my $globals, '<', '/etc/asterisk/fonality/globals.conf' || return($user, $pass);
	while (my $line = <$globals>)
	{
		if ($line =~ /^IAXINFO/)
		{
			if ($line =~ /IAXINFO\d?=(s\d+):([^;\s]+)/)
			{
				($user, $pass) = ($1, Digest::MD5::md5_hex($2));
				$user =~ s/[xi]$//;
			}
			last;
		}
	}

	close $globals;
	return($user, $pass);
}
